// Fill out your copyright notice in the Description page of Project Settings.


#include "NetworkConnector.h"



#include "HttpModule.h"
#include "Interfaces/IHttpRequest.h"
#include "Interfaces/IHttpResponse.h"

#include "Dom/JsonObject.h"
#include "Dom/JsonValue.h"
#include "Serialization/JsonWriter.h"
#include "Serialization/JsonSerializer.h"

void UNetworkConnector::SendResultsToServer(FSimpleNetworkFinishedDelegate OnComplete, FResultsStructure Results)
{
	auto RequestObject = MakeShared<FJsonObject>();
	RequestObject->SetStringField("HMDName", Results.HMDName);
	RequestObject->SetStringField("HMDRevision", Results.HMDRevision);
	RequestObject->SetNumberField("HorizontalFOV", Results.HorizontalFOV);
	RequestObject->SetNumberField("VerticalFOV", Results.VerticalFOV);
	RequestObject->SetNumberField("TextReadability", Results.TextReadability);
	
	if (Results.Modded)
	{
		RequestObject->SetNumberField("modded", 1);
	}

	auto JsonOut = FString();
	auto Writer = TJsonWriterFactory<>::Create(&JsonOut);

	auto Ok = FJsonSerializer::Serialize(RequestObject, Writer);
	if (!Ok) {
		UE_LOG(LogTemp, Error, TEXT("Request object serialization failed"));
	}

	auto Request = FHttpModule::Get().CreateRequest();
	Request->SetHeader("Content-Type", "application/json");
	Request->SetVerb("POST");

	Request->SetContentAsString(JsonOut);
	Request->SetURL("88.86.119.102:60001/result");

	Request->OnProcessRequestComplete()
		.BindLambda([OnComplete](FHttpRequestPtr requestPtr, FHttpResponsePtr ResponsePtr, bool bWasSuccessful) {
		// unknown failure
		if (!ResponsePtr || !bWasSuccessful) {
			UE_LOG(LogTemp, Error, TEXT("Network request failed for unknown reason. The server is probably down."));
			OnComplete.ExecuteIfBound(false, -1);
			return;
		}

		auto ResponseCode = ResponsePtr->GetResponseCode();
		if (ResponseCode != 200) {
			UE_LOG(LogTemp, Error, TEXT("Wrong network response code: %i"), ResponseCode);
		}


		OnComplete.ExecuteIfBound(true, ResponseCode);
		return;
	});

	Request->ProcessRequest();
}
